#!/usr/bin/env python

import random, os.path
import sys

#import basic pygame modules
import pygame
from pygame.locals import *

#see if we can load more than standard BMP
if not pygame.image.get_extended():
    raise SystemExit("Sorry, extended image module required")

sys.stderr = open("logfile.txt","w")

#game constants
SCREENRECT     = Rect(0, 0, 900, 539)

FPS = 30
FRAMES_BETWEEN_PLAYER_COOS = 10 #FPS*3/4
FRAMES_BETWEEN_PLAYER_FLUTTERS = FPS*3/4
MAX_FRAMES_BETWEEN_PECK = FPS*4

GAMEPLAY_TOP = 317
GAMEPLAY_BOTTOM = (GAMEPLAY_TOP+65)
GAMEPLAY_WIDTH = 10

BGPIGEON_TOP = GAMEPLAY_TOP -GAMEPLAY_WIDTH
FGPIGEON_TOP = GAMEPLAY_TOP + GAMEPLAY_WIDTH

MAN_X = 638

SUN_X = 617
SUN_START_Y = 160

SECONDS_TO_SUNSET = 45
SUNSET_Y = 305

COO_SUNUP = 2
LANDING_TIME = FPS*4

main_dir = os.path.split(os.path.abspath(sys.argv[0]))[0]

layer_player = None
pct_night = 0.0
the_end = None
angry_channel = None

n_angry_pigeons = 0

sounds = {}

def datafile(filename):
	components = filename.split("/")
	return os.path.join(components)

def lerp(start, end, amt):
	return (end-start)*amt + start

def get_rand_pigeon_coords():
    return [
        0 + random.randrange(540),
        BGPIGEON_TOP + random.randrange(40)
    ]

def rand_angry_sound():
    #print "playing angry sound!!!!"
    return random.choice(sounds['angry'])

def rand_flap_sound():
    return random.choice(sounds['flap'])

def rand_tie_sound():
    return random.choice(sounds['tie'])

def coo_sound_at_power(p):
    choices = ()
    if p == 1:
        choices = ('coo1', 'coo2', 'coo3')
    elif p == 2:
        choices = ('coo4', 'coo5', 'coo6')
    elif p == 3:
        choices = ('coo7', 'coo8', 'coo9')
    elif p == 4:
        choices = ('coo10', 'coo11', 'coo12')
    elif p == 5:
        return sounds['coo12']
    return sounds[random.choice(choices)]

def load_image(file):
    "loads an image, prepares it for play"
    file = os.path.join(main_dir, *(datafile(file)))
    try:
        surface = pygame.image.load(file)
    except pygame.error:
        raise SystemExit('Could not load image "%s" %s'%(file, pygame.get_error()))

    if file.endswith(".png"):
        return surface.convert_alpha()
    else:
        surface.set_colorkey(pygame.Color(255,0,255,255))
        return surface.convert()

def load_images(*files):
    imgs = []
    for file in files:
        imgs.append(load_image(file))
    return imgs

class dummysound:
    name = None
    set_volume = None

    def __init__(self, name):
        self.name = name
    def play(self):
        print 'not playing sound: ' + self.name
    def get_num_channels(self):
        return 1

def load_sound(filename):
    if not pygame.mixer:
        print 'no mixer!'
        return dummysound(filename)
    file = os.path.join(main_dir, datafile(filename))
    if not os.path.exists(file):
        print "doesn't exist (%s)" % file
        return dummysound(filename)
    try:
        sound = pygame.mixer.Sound(file)
        return sound
    except pygame.error:
        print ('Warning, unable to load, %s' % file)
    return dummysound(filename)

# each type of game object gets an init and an
# update function. the update function is called
# once per frame, and it is when each object should
# change it's current position and state. the Player
# object actually gets a "move" function instead of
# update, since it is passed extra information about
# the keyboard

class AnimState:
    def __init__(self, name):
        self.name = name
        self.images = []
        self.timing = []

    def frame(self, surface, time=-1):
        self.images.append(surface)

        if len(self.timing) > 0:
            time += self.timing[-1]
        self.timing.append(time)

    def image_at_frame(self, frame):
        if len(self.images) == 1 and self.timing[0] == -1:
            return self.images[0]

        found = None
        for index, lastframe in enumerate(self.timing):
            if frame <= lastframe:
                found = index
                break
        if found == None:
            #print "%s[NONE] for %d" % (self.name, frame)
            return None
        else:
            #print "%s[%d] for %d" % (self.name, found, frame) 
            return self.images[found]

class Animated(pygame.sprite.DirtySprite):
    states = []
    transitions = {}
    frame = 0
    facing = 0
    scale = 1.0

    def __init__(self):
        pygame.sprite.DirtySprite.__init__(self)
        self.current_state = None
        self.facing = 1
        self.alpha = 255
        self.rect = self.states[0].images[0].get_rect()
        self.update()

    @classmethod
    def addState(c, state):
        c.states.append(state)

    @classmethod
    def addTransition(c, fromname, toname):
        fromstate = None
        tostate = None
        for s in c.states:
            if s.name == fromname:
                fromstate = s
            if s.name == toname:
                tostate = s

        if fromstate == None or tostate == None:
            raise Error("couldn't find states for transition")

        c.transitions[fromstate] = tostate

    def isState(self, name):
        st = None
        for s in self.states:
            if s.name == name:
                st = s
                break

        if st == None:
            raise Error("no state named " + s)

        return (self.current_state == st)

    def setState(self, name):
        for s in self.states:
            if s.name == name:
                self.current_state = s
                self.frame = 0
                return
        raise Error("no state named " + s)

    def on_transition_to(self, statename):
        pass

    def update(self):
        self.frame += 1
        if self.current_state == None:
            self.current_state = self.states[0]
        img = self.current_state.image_at_frame(self.frame)
        if img == None:
            self.frame = 0
            if self.current_state in self.transitions:
                self.current_state = self.transitions[self.current_state]
                self.on_transition_to(self.current_state.name)
            img = self.current_state.image_at_frame(self.frame)
        if self.facing < 0:
            img = pygame.transform.flip(img,1,0)
        if self.scale != 1.0:
            x, y = img.get_size
            img = pygame.transform.smoothscale(img,x,y)
        self.image = img

layer_gameplay = None
layer_bgpigeons = None
layer_fgpigeons = None



class Pigeon(Animated):
    speed = 1

def success():
    global ending
    global man
    global tie
    global blacktie
    global layer_fgpigeons
    global layer_gameplay

    #print "good ending!"
    ending = True
    tie.setState("END")
    blacktie.setState("END")
    layer_fgpigeons.remove(tie)
    layer_fgpigeons.remove(blacktie)
    layer_gameplay.remove(man)
    layer_fgpigeons.add(man)
    man.hmm()

class PlayerPigeon(Pigeon):
    states = []
    transitions = {}
    y_cycle = [0,-1,-2,-3,-4,-5,-6,-7,-8,-7,-6,-5,-4,-3,-2,-1,0]
    y_frame = None
    y_adjust = None
    
    flutter_countdown = 0
    coo_countdown = 0
    is_walking = False
    is_busy = False
    base_y = None

    def __init__(self):
        Pigeon.__init__(self)
        self.setState("STAND")
        self.rect = self.image.get_rect(left=SCREENRECT.left, bottom=GAMEPLAY_BOTTOM)
        self.base_y = self.rect.top
        self.y = self.rect.top
        
    def coo(self):
        global countup

        if self.is_walking:
            return

        if self.coo_countdown > 0:
            #print 'coo countdown needed. %d' % self.coo_countdown
            return

        self.setState("COO")
        self.y_adjust = -9
        self.is_busy = True
        self.coo_countdown = FRAMES_BETWEEN_PLAYER_COOS

        coo_power = int(self.rect.right / (MAN_X / 4)) + 1

        #print 'coo power is %d' % (coo_power)

        coo_sound_at_power(coo_power).play()

        if n_angry_pigeons > 0:
            #print 'angry pigeons!'
            return
        else:
            # coo effects do
            #print 'pre coo %d' % countup
            countup = countup - (COO_SUNUP * (coo_power/2))
            if countup < 0:
                countup = 0
            #print 'post coo %d' % countup
            dirtysky = True

            if coo_power == 5:
                success()

        self.update()
    
    def on_transition_to(self, statename):
        if statename == "STAND":
            self.is_busy = False
            self.y_adjust = 0

    def scatter(self):
        if self.is_walking:
            return

        if self.flutter_countdown > 0:
            #print 'flutter countdown needed'
            return

        self.setState("SCATTER")
        self.y_frame = 0
        self.is_busy = True
        self.flutter_countdown = FRAMES_BETWEEN_PLAYER_FLUTTERS

    def update(self):
        if self.is_walking and not self.isState("WALK"):
            self.setState("WALK")
        elif (not self.is_walking) and self.isState("WALK"):
            self.setState("STAND")

        Pigeon.update(self)
        if self.flutter_countdown > 0:
            self.flutter_countdown -= 1
        if self.coo_countdown > 0:
            self.coo_countdown -= 1
        self.is_walking = False

        if self.y_adjust != None:
            if self.y_adjust == 0:
                self.y = self.base_y
                self.rect.top = self.y
                self.y_adjust = None
            else:
                self.y = self.base_y + self.y_adjust
                self.rect.top = self.y

        if self.y_adjust == None and self.y_frame != None:
            if self.y_frame == 3:
                collisions = pygame.sprite.spritecollide( self, layer_gameplay,  False, pygame.sprite.collide_rect_ratio(2.0))
                collisions += pygame.sprite.spritecollide(self, layer_bgpigeons, False, pygame.sprite.collide_rect_ratio(1.33))
                collisions += pygame.sprite.spritecollide(self, layer_fgpigeons, False, pygame.sprite.collide_rect_ratio(1.33))
                for c in collisions:
                    if isinstance(c, OtherPigeon):
                        c.scatter()

            if self.y_frame >= len(self.y_cycle):
                self.y_frame = None
                self.y = self.base_y
                self.is_busy = False
            else:
                self.y = self.base_y + self.y_cycle[self.y_frame]*3
                self.y_frame += 1
            self.rect.top = self.y

    def move(self, direction):
        if self.is_busy:
            return

        if direction:
            self.is_walking = True
            self.facing = direction
        
            movement = direction*self.speed
            self.rect.move_ip(movement, 0)
            self.rect = self.rect.clamp(SCREENRECT)

            collisions = pygame.sprite.spritecollideany(self, layer_gameplay)
            if collisions != None:
                #print 'collide!'
                #print(collisions)
                self.rect.move_ip(-movement, 0)
        
        #if direction < 0:
        #    self.image = self.images[0]
        # elif direction > 0:
        #    self.image = self.images[1]
        #self.rect.top = self.origtop - (self.rect.left//self.bounce%2)

class OtherPigeon(Pigeon):
    speed = 10
    time_to_peck = 0
    states = []
    transitions = {}
    is_scattering= False
    is_on_ground = True
    is_soaring = False
    is_landing = False
    scatter_frame = None
    land_frame = None
    land_targetrect = None
    land_sourcerect = None


    def __init__(self, leftpos, toppos):
        Pigeon.__init__(self)
        self.rect = self.image.get_rect(left=leftpos, top=toppos)
        self.facing = random.choice((-1,1))
        self.setState("PECK")
        
        #if self.facing < 0:
        #    self.rect.right = SCREENRECT.right

    
    def on_transition_to(self, statename):
        if statename == "LAND":
            check = random.randrange(2)
            #print 'check land is %d' % (check)
            if check != 1:
                rand_angry_sound().play()
                self.setState("SOAR")
                return

            global n_angry_pigeons
            n_angry_pigeons -= 1

            #print 'tx to land'
            # this is 1x FPS
            self.is_landing = True
            self.is_soaring = False
            self.land_frame = 0
            self.land_sourcerect = self.rect

            target = get_rand_pigeon_coords()

            self.land_targetrect = pygame.Rect(
                target[0],
                target[1],
                self.rect.width,
                self.rect.height
            )
            if self.land_sourcerect.left <= self.land_targetrect.left:
                self.facing = 1
            else:
                self.facing = -1

            self.is_scattering = False

        elif statename == "SOAR":
            if self.rect.top <= self.scattergoal:
                #print 'hit scattergoal of %d' % (self.scattergoal)
                self.scatter_frame = None
            else:
                self.setState("FLAP")
                return
            #print 'transition to soar'
            # this is 3/2 FPS
            self.is_scattering = False
            self.is_soaring = False

        elif statename == "PECK":
            # landed
            self.update_layer()
            self.is_on_ground = True
            self.is_landing = False


    def update_layer(self):
        # update layer
        for g in self.groups():
            g.remove(self)

        if self.rect.top < BGPIGEON_TOP:
            layer_bgpigeons.add(self)

        elif self.rect.top > FGPIGEON_TOP:
            layer_fgpigeons.add(self)

        else:
            layer_gameplay.add(self)

    def update(self):
        if self.is_on_ground:
            self.time_to_peck -= 1
            if self.time_to_peck <= 0:
                self.setState("PECK")
                self.time_to_peck = FPS/15 + random.randrange(FPS*4)

        elif self.is_scattering:

            self.facing = 1
            if self.scatter_frame == 0:
                self.rect.move_ip(self.facing * 2, -2)
            elif self.scatter_frame == 1:
                self.rect.move_ip(self.facing * 3, -6)
            elif self.scatter_frame >= 2:
                self.rect.move_ip(self.facing * 4, -3 -random.randrange(10))

            self.rect = self.rect.clamp(SCREENRECT)

            if self.rect.top <= self.scattergoal:
                #print 'hit scattergoal of %d' % (self.scattergoal)
                self.scatter_frame = None
                self.setState("SOAR")
                self.on_transition_to("SOAR")

            if self.scatter_frame != None:
                self.scatter_frame += 1

        elif self.current_state.name == "SOAR":
            dir = self.facing
            movement = dir*self.speed
            self.rect.move_ip(movement, random.choice((1,0)))
            self.rect = self.rect.clamp(SCREENRECT)

            if self.rect.left == SCREENRECT.left:
                self.facing = -self.facing
            elif self.rect.right >= SCREENRECT.right*7/8:
                self.facing = -self.facing

        elif self.current_state.name == "LAND":

            land_progress = float(self.land_frame) / float(LANDING_TIME)

            oldleft = self.rect.left
            self.rect.left = int(lerp(self.land_sourcerect.left, self.land_targetrect.left, land_progress * 0.5))

            self.rect.top  = int(lerp(self.land_sourcerect.top,  self.land_targetrect.top,  land_progress * 0.5))

            collisions = pygame.sprite.spritecollideany(self, layer_player)
            if collisions != None:
                #print 'landing bird collides with layer_player!'
                #print(collisions)
                self.rect.left = oldleft

            if land_progress >= 0.4:
                self.setState("PECK")
                self.on_transition_to("PECK")

            self.land_frame += 1

        #else:
            #print 'unmatched state %s' % self.current_state.name



        Pigeon.update(self)

    def scatter(self):
        global n_angry_pigeons

        rand_angry_sound().play()
        rand_flap_sound().play()
        n_angry_pigeons += 1
        self.setState("FLAP")
        self.is_scattering = True
        self.is_on_ground = False
        self.scatter_frame = 0
        self.scattergoal = random.randrange(200) + 25  

class Bread(pygame.sprite.Sprite):
    images = []

    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.image = self.images[0]
        self.rect = self.image.get_rect()


class Man(Animated):
    speed = 0
    states = []
    transitions = {}

    def __init__(self, leftpos, toppos):
        Animated.__init__(self)
        self.rect = self.states[0].images[0].get_rect(left=leftpos, top=toppos)
        self.update()
        self.setState("THINK")

    def on_transition_to(self, statename):
        global the_end

        #print "transition to " + statename

        if statename == "GONE":
            the_end.start()
        elif statename == "OK":
            pygame.time.wait(3000)
            the_end.start()


    def hmm(self):
        #print self.rect

        self.setState("HMM")
        self.rect.top  -= 18
        self.rect.left -= 80
        #print self.rect
        self.update()
        #print self.rect


blacktie = None

class Tie(Animated):
    speed = 0
    states = []
    transitions = {}
    time_to_flap = 1
    alpha = 255
    fading = False
    fadein = False

    def __init__(self, leftpos, toppos):
        Animated.__init__(self)
        self.rect = self.states[0].images[0].get_rect(left=leftpos, top=toppos)
        self.update()
        self.setState("IDLE")

    def update(self):
        global blacktie

        if self.fadein:
            self.alpha += 4
            # black tie white noise
            self.image.set_alpha(self.alpha)
            self.dirty = 1
            return

        if pct_night >= 1.05:
            return

        if pct_night >= 0.8 and self != blacktie:
            global layer_fgpigeons

            if not self.isState("END"):
                self.setState("END")

            #print blacktie.groups()

            if pct_night > 0.81 and blacktie.groups() == [] and not ending:
                layer_fgpigeons.add(blacktie)

            elif pct_night <= 0.81 and blacktie.groups() != []:
                layer_fgpigeons.remove(blacktie)

            x = 10.0 * (pct_night - 0.8)
            #print 'fading tie'
            self.alpha = int(lerp(1.0, 0.0, x) * 255)
            blacktie.alpha = 255-self.alpha
            self.image.set_alpha(self.alpha)
            blacktie.image.set_alpha(blacktie.alpha)
            self.dirty = 1
            blacktie.dirty = 1
            blacktie.update()

        else:
            if self.isState("END"):
                self.setState("IDLE")

            if self.isState("IDLE"):
                self.time_to_flap -= 1
            if self.time_to_flap <= 0:
                tgtstate = random.choice(("SMALLFLAP","SMALLFLAP","BIGFLAP"))
                if tgtstate == "SMALLFLAP":
                    self.time_to_flap = random.randrange(FPS)
                else:
                    rand_tie_sound().play()
                    self.time_to_flap = random.randrange(FPS*4)
                self.setState(tgtstate)

        Animated.update(self)

    def set_alpha(self, a):
        self.alpha = a

    def start_fadein(self):
        self.fadein = True

    def start_fade(self):
        self.fading = True


class BGComponent(pygame.sprite.DirtySprite):

    def __init__(self, imagename, x, y):
        pygame.sprite.DirtySprite.__init__(self)
        self.image = load_image(imagename)
        self.rect = self.image.get_rect(top = y, left = x)
        self.dirty = 1

    def setAlpha(self, alpha):
        self.dirty = 1
        self.image.set_alpha(int(alpha*255))

    def update(self):
        self.dirty = 0


dirtysky = False

sunframe = 0
sunticks = (SECONDS_TO_SUNSET*FPS) / (SUNSET_Y - SUN_START_Y)
countup = 0

BG = None
clock = None
man = None

tie = None

def suntick():
    global sunframe
    global countup
    global dirtysky
    global sunticks
    global pct_night
    global ending
    global man
    global tie
    global blacktie

    sunframe += 1
    if sunframe == sunticks:
        dirtysky = True
        sunframe = 0
        countup += 1
        pct_night = (float(countup) / float(SUNSET_Y - SUN_START_Y))

        if pct_night >= 1.0 and not ending:
            #print "end!"
            ending = True
            man.setState("FALL")
            tie.setState("END")
            blacktie.setState("END")
            global layer_fgpigeons
            layer_fgpigeons.remove(tie)
            layer_fgpigeons.remove(blacktie)


class Sun(BGComponent):

    def __init__(self):
        BGComponent.__init__(self, "data/sun.png", SUN_X, SUN_START_Y)

    def update(self):
        BGComponent.update(self)
        if dirtysky:
            self.rect.top = int(lerp(SUN_START_Y, SUNSET_Y, pct_night))
            self.dirty = 1

class Fader(BGComponent):

    def __init__(self, img, x, y, fade_from, fade_to):
        BGComponent.__init__(self, img, x, y)
        self.fade_to = fade_to
        self.fade_from = fade_from
        self.setAlpha(fade_from)

    def update(self):
        BGComponent.update(self)
        if dirtysky:
            if pct_night >= 0.7:
                x = 10.0 * (pct_night - 0.7)
                self.setAlpha(lerp(self.fade_from, self.fade_to, x * 0.2))
                self.dirty = 1

class LateFader(Fader):
    def update(self):
        BGComponent.update(self)
        if dirtysky:
            if pct_night >= 1.9:
                x = 10.0 * (pct_night - 1.0)
                self.setAlpha(lerp(self.fade_from, self.fade_to, x))
                self.dirty = 1

class Glow(BGComponent):
    time_to_fade = FPS

    start_x = SCREENRECT.right * 3/4
    x = 0
    start_y = SCREENRECT.bottom * 3/4
    y = 0

    tgt_x = 900-606
    tgt_y = 540-457

    def __init__(self):
        BGComponent.__init__(self, "data/glow.png", self.start_x, self.start_y)
        self.x = self.start_x
        self.y = self.start_y

    def update(self):
        BGComponent.update(self)
        if dirtysky:
            if pct_night >= 0.7:
                r = 10.0 * (pct_night - 0.7)
                self.x = lerp(self.start_x, self.tgt_x, r * 0.2)
                self.y = lerp(self.start_y, self.tgt_y, r * 0.2)

                self.rect.top = self.y
                self.rect.left = self.x

                self.dirty = 1

class Intro(BGComponent):
    time_to_stay = FPS*3
    time_to_fade = FPS 
    frame = 0
    fading = False
    alpha = 255

    def __init__(self):
        BGComponent.__init__(self, "data/bmp/intro.bmp", 0,0)

    def update(self):
        if self.alpha <= 0:
            return
        if self.fading == True:
            #print "churning alpha"
            self.alpha -= 8
            self.setAlpha(float(self.alpha) / 255.0)
        
        self.frame += 1

        if self.frame == self.time_to_stay:
            self.fading = True
            global introing
            introing = False
            #print "fade now"


class TheEnd(BGComponent):
    time_to_fade = FPS
    frame = 0
    go = False

    def __init__(self):
        BGComponent.__init__(self, "data/bmp/end.bmp", 0,0)
        self.setAlpha(0.0)

    def start(self):
        self.go = True

    def update(self):
        if not self.go:
            return

        self.frame += 1

        if self.frame == self.time_to_fade:
            self.setAlpha(1.0)
        elif self.frame > self.time_to_fade:
            if pygame.mixer:
                pygame.mixer.music.fadeout(3000)
            pygame.time.wait(5000)
            pygame.quit()
            quit()
        else:
            self.setAlpha(float(self.frame) / float(self.time_to_fade))


screen = None
layers = None
ending = False
introing = True

def main(winstyle = 0):
    global dirtysky
    global layer_gameplay
    global layer_bgpigeons
    global layer_player
    global layer_fgpigeons
    global blackscreen
    global screen
    global clock
    global man
    global BG
    global layers
    global countup
    global ending
    global tie
    global angry_channel
    global sounds
    global the_end
    global introing
    global blacktie

    # Initialize pygame
    pygame.init()

    # Set the display mode
    winstyle = pygame.NOFRAME | pygame.HWSURFACE | pygame.DOUBLEBUF
    bestdepth = pygame.display.mode_ok(SCREENRECT.size, winstyle, 32)
    screen = pygame.display.set_mode(SCREENRECT.size, winstyle, bestdepth)

    if pygame.mixer:
        pygame.mixer.pre_init(44100)
        if not pygame.mixer.get_init():
            print ('Warning, no sound')
            pygame.mixer = None

    for i in range(12):
        s = i + 1
        sounds['coo' + str(s)] = load_sound("data/sound/PIGEONCOO" + str(s) + ".ogg")

    sounds['angry'] = [load_sound("data/sound/ANGRYBIRDS" + str(i+1) + ".ogg") for i in range(3)]

    for s in sounds['angry']:
		# fix for dummysound - no sound card available?
		if s.set_volume != None:
			s.set_volume(0.75)

    sounds['flap'] = [load_sound("data/sound/FLAPPING" + str(i+1) + ".ogg") for i in range(3)]
    sounds['tie'] = [load_sound("data/sound/TIEFLAP" + str(i+1) + ".ogg") for i in range(3)]

    if pygame.mixer:
        music = os.path.join(main_dir, "data/sound/pigeondrone2.ogg")
        pygame.mixer.music.load(music)
        pygame.mixer.music.play(-1)

    blackscreen = pygame.Surface(SCREENRECT.size)
    blackscreen.fill(pygame.Color("black"))
    blackscreen.convert()

    #Load images, assign to sprite classes
    #(do this before the classes are used, after screen setup)
    bg_sky      = Fader("data/bmp/background.bmp",    0,  0, 1.0, 0.0)
    bg_nightsky = Fader("data/bmp/bg2.bmp",           0,  0, 0.0, 1.0)
    bg_sun   = Sun()
    bg_horiz = BGComponent("data/bmp/horizon.bmp",       0,    85)
    bg_roof       = Fader("data/bmp/rooftop.bmp",        0,   339, 1.0, 0.0)
    bg_nightroof  = Fader("data/bmp/rooftop2.bmp",       0,   339, 0.0, 1.0)
    bg_sign       = BGComponent("data/breadSign_off.png",  0, 540-57)
    bg_nightsign  = Fader(      "data/bmp/signglow.bmp",       0, 540-53, 0.0, 1.0)
    bg_glow   = Glow()

    BG = pygame.sprite.OrderedUpdates()
    BG.add(bg_sky)
    BG.add(bg_nightsky)
    BG.add(bg_sun)
    BG.add(bg_horiz)
    BG.add(bg_roof)
    BG.add(bg_nightroof)
    BG.add(bg_sign)
    
    BG.add(bg_nightsign)
    BG.add(bg_glow)

    #img = load_image("data/player1.gif')
    #Player.images = [img, pygame.transform.flip(img, 1, 0)]
    #img = load_image("data/explosion1.gif')
    #Explosion.images = [img, pygame.transform.flip(img, 1, 1)]
    #Alien.images = load_images("data/alien1.gif', "data/alien2.gif', "data/alien3.gif')
    #Bomb.images = [load_image("data/bomb.gif')]
    #Shot.images = [load_image("data/shot.gif')]

    #decorate the game window
    icon = pygame.transform.scale(load_image("data/png/pigWalk_2.png"), (32, 32))
    pygame.display.set_icon(icon)
    pygame.display.set_caption('pigeon')
    #pygame.mouse.set_visible(0)

    #load the sound effects
    #boom_sound = load_sound('boom.wav')
    #shoot_sound = load_sound('car_door.wav')
    #if pygame.mixer:
    #    music = os.path.join(main_dir, 'data', 'house_lo.wav')
    #    pygame.mixer.music.load(music)
    #    pygame.mixer.music.play(-1)

    st = AnimState("WALK")
    st.frame(load_image("data/png/pigWalk_1.png"), FPS/15*2)
    st.frame(load_image("data/png/pigWalk_2.png"), FPS/15*2)
    st.frame(load_image("data/png/pigWalk_3.png"), FPS/15*2)
    PlayerPigeon.addState(st)

    st = AnimState("STAND")
    st.frame(load_image("data/png/pigWalk_2.png"))
    PlayerPigeon.addState(st)

    st = AnimState("BLACK")
    st.frame(load_image("data/png/pigBlackWalk_2.png"))
    OtherPigeon.addState(st)

    st = AnimState("SOAR")
    st.frame(load_image("data/png/pigSoar.png"))
    PlayerPigeon.addState(st)

    st = AnimState("SCATTER")
    st.frame(load_image("data/png/pigLand.png"), FPS/10)
    st.frame(load_image("data/png/pigFlap1.png"), FPS/15)
    st.frame(load_image("data/png/pigFlap2.png"), FPS/15)
    st.frame(load_image("data/png/pigFlap1.png"), FPS/15)
    st.frame(load_image("data/png/pigFlap2.png"), FPS/15)
    st.frame(load_image("data/png/pigFlap1.png"), FPS/15)
    st.frame(load_image("data/png/pigFlap2.png"), FPS/15)
    st.frame(load_image("data/png/pigLand.png"), FPS/10)
    PlayerPigeon.addState(st)
    PlayerPigeon.addTransition("SCATTER", "STAND")

    st = AnimState("FLAP")
    st.frame(load_image("data/png/pigFlap1.png"), FPS/10)
    st.frame(load_image("data/png/pigFlap2.png"), FPS/10)
    PlayerPigeon.addState(st)

    st = AnimState("LAND")
    st.frame(load_image("data/png/pigLand.png"))
    PlayerPigeon.addState(st)

    st = AnimState("COO")
    st.frame(load_image("data/png/pigCoo.png"), FPS/8)
    PlayerPigeon.addState(st)
    PlayerPigeon.addTransition("COO", "STAND")

    st = AnimState("PECK")
    st.frame(load_image("data/png/pigPeck1.png"), FPS/15)
    st.frame(load_image("data/png/pigPeck2.png"), FPS/15)
    st.frame(load_image("data/png/pigPeck1.png"), FPS/15)
    PlayerPigeon.addState(st)
    PlayerPigeon.addTransition("PECK", "STAND")

    st = AnimState("WALK")
    st.frame(load_image("data/png/pigBlackWalk_1.png"), FPS/15)
    st.frame(load_image("data/png/pigBlackWalk_2.png"), FPS/15)
    st.frame(load_image("data/png/pigBlackWalk_3.png"), FPS/15)
    OtherPigeon.addState(st)

    st = AnimState("STAND")
    st.frame(load_image("data/png/pigBlackWalk_2.png"))
    OtherPigeon.addState(st)

    st = AnimState("SOAR")
    st.frame(load_image("data/png/pigBlackSoar.png"), FPS*3/4)
    OtherPigeon.addState(st)

    st = AnimState("FLAP")
    st.frame(load_image("data/png/pigBlackFlap1.png"), FPS/15)
    st.frame(load_image("data/png/pigBlackFlap2.png"), FPS/15)
    st.frame(load_image("data/png/pigBlackFlap1.png"), FPS/15)
    st.frame(load_image("data/png/pigBlackFlap2.png"), FPS/15)
    st.frame(load_image("data/png/pigBlackFlap1.png"), FPS/15)
    st.frame(load_image("data/png/pigBlackFlap2.png"), FPS/15)
    st.frame(load_image("data/png/pigBlackFlap1.png"), FPS/15)
    st.frame(load_image("data/png/pigBlackFlap2.png"), FPS/15)
    OtherPigeon.addState(st)
    OtherPigeon.addTransition("FLAP", "SOAR")

    st = AnimState("LAND")
    st.frame(load_image("data/png/pigBlackLand.png"), LANDING_TIME)
    OtherPigeon.addState(st)
    OtherPigeon.addTransition("SOAR", "LAND")

    st = AnimState("PECK")
    st.frame(load_image("data/png/pigBlackPeck1.png"), FPS/10)
    st.frame(load_image("data/png/pigBlackPeck2.png"), FPS*10)
    OtherPigeon.addState(st)
    OtherPigeon.addTransition("LAND", "PECK")

    st = AnimState("THINK")
    st.frame(load_image("data/man/man.png"))
    Man.addState(st)

    st = AnimState("HMM")
    st.frame(load_image("data/png/success/s1.png"), FPS/10)
    st.frame(load_image("data/png/success/s2.png"), FPS/10)
    st.frame(load_image("data/png/success/s3.png"), FPS/10)
    st.frame(load_image("data/png/success/s4.png"), FPS/10)
    st.frame(load_image("data/png/success/s5.png"), FPS/10)
    st.frame(load_image("data/png/success/s6.png"), FPS/10)
    st.frame(load_image("data/png/success/s7.png"), FPS/10)
    st.frame(load_image("data/png/success/s8.png"), FPS/10)
    st.frame(load_image("data/png/success/s9.png"), FPS/10)
    st.frame(load_image("data/png/success/s10.png"), FPS/10)
    st.frame(load_image("data/png/success/s11.png"), FPS/10)
    st.frame(load_image("data/png/success/s12.png"), FPS/10)
    st.frame(load_image("data/png/success/s13.png"), FPS/10)
    st.frame(load_image("data/png/success/s14.png"), FPS/10)
    st.frame(load_image("data/png/success/s15.png"), FPS/10)
    st.frame(load_image("data/png/success/s16.png"), FPS/10)
    st.frame(load_image("data/png/success/s17.png"), FPS/10)
    Man.addState(st)

    st = AnimState("OK")
    st.frame(load_image("data/png/success/s17.png"))
    Man.addState(st)
    Man.addTransition("HMM", "OK")

    st = AnimState("FALL")
    st.frame(load_image("data/man/fall/1.png"), FPS/10)
    st.frame(load_image("data/man/fall/2.png"), FPS/10)
    st.frame(load_image("data/man/fall/3.png"), FPS/10)
    st.frame(load_image("data/man/fall/4.png"), FPS/10)
    st.frame(load_image("data/man/fall/5.png"), FPS/10)
    st.frame(load_image("data/man/fall/6.png"), FPS/10)
    st.frame(load_image("data/man/fall/7.png"), FPS/10)
    st.frame(load_image("data/man/fall/8.png"), FPS/10)
    st.frame(load_image("data/man/fall/9.png"), FPS/10)
    Man.addState(st)

    st = AnimState("GONE")
    st.frame(pygame.image.frombuffer("0", (1,1), "P"))
    Man.addState(st)
    Man.addTransition("FALL", "GONE")

    st = AnimState("IDLE")
    st.frame(load_image("data/man/tie4.png"))
    Tie.addState(st)

    st = AnimState("END")
    st.frame(load_image("data/bmp/tie4_white.bmp"))
    Tie.addState(st)

    st = AnimState("SMALLFLAP")
    st.frame(load_image("data/man/tie5.png"), FPS/10)
    Tie.addState(st)
    Tie.addTransition("SMALLFLAP", "IDLE")

    st = AnimState("BLACK")
    st.frame(load_image("data/bmp/tie4_black.bmp"))
    Tie.addState(st)

    st = AnimState("BIGFLAP")
    st.frame(load_image("data/man/tie1.png"), FPS/10)
    st.frame(load_image("data/man/tie2.png"), FPS/10)
    st.frame(load_image("data/man/tie3.png"), FPS/6)
    st.frame(load_image("data/man/tie2.png"), FPS/10)
    st.frame(load_image("data/man/tie3.png"), FPS/6)
    st.frame(load_image("data/man/tie2.png"), FPS/10)
    st.frame(load_image("data/man/tie1.png"), FPS/10)
    Tie.addState(st)
    Tie.addTransition("BIGFLAP", "IDLE")
    
    layer_bgpigeons = pygame.sprite.RenderUpdates()
    layer_gameplay  = pygame.sprite.RenderUpdates()
    layer_player    = pygame.sprite.RenderUpdates()
    layer_fgpigeons = pygame.sprite.RenderUpdates()
    layer_black     = pygame.sprite.RenderUpdates()

    layers = [layer_bgpigeons, layer_gameplay, layer_player, layer_fgpigeons, layer_black]

    fg_black = LateFader("data/bmp/black.bmp", 0, 0, 0.0, 1.0)
    layer_black.add(fg_black)

    the_end = TheEnd()
    layer_black.add(the_end)

    intro = Intro()
    layer_black.add(intro)


    #Create Some Starting Values
    #global score
    #alienreload = ALIEN_RELOAD
    #kills = 0
    clock = pygame.time.Clock()

    #initialize our starting sprites
    #global SCORE
    #Alien() #note, this 'lives' because it goes into a sprite group

    coordlist = []
    for i in range(20):
        coordlist.append(get_rand_pigeon_coords())

    for coords in coordlist:
        op = OtherPigeon(coords[0], coords[1])
        op.update_layer()
    
    player = PlayerPigeon()
    layer_player.add(player)

    tie = Tie(696, 119)
    layer_fgpigeons.add(tie)

    blacktie = Tie(696,119)
    
    blacktie.set_alpha(0)
    blacktie.setState("BLACK")

    man = Man(MAN_X, 61)
    layer_gameplay.add(man)
    
    bgbuffer = pygame.Surface(SCREENRECT.size)

    first_bg_blit = True

    ending = False

    while True:

        if not ending and not introing:
            #get input
            for event in pygame.event.get():
                if event.type == QUIT or \
                    (event.type == KEYDOWN and event.key == K_ESCAPE):
                        return
        
        keystate = pygame.key.get_pressed()

        BG.clear(screen, bgbuffer)

        suntick()
        BG.update()

        # reset every frame after update
        dirtysky = False

        dirty = []
        drawbg = False
        start_drawing = False

        if first_bg_blit == True:
            drawbg = True
            first_bg_blit = False
        else:
            # clear/erase the last drawn sprites
            for s in BG.sprites():
                if s.dirty:
                    drawbg = True
                    break

        if drawbg:
            BG.draw(bgbuffer)
            screen.blit(bgbuffer, SCREENRECT)

        dirty = []

        for layer in layers:
            layer.clear(screen, bgbuffer)
            layer.update()

        if not ending and not introing:
            #handle player input
            direction = (keystate[K_RIGHT] or keystate[K_d]) - (keystate[K_LEFT] or keystate[K_a])
            player.move(direction)

            if keystate[K_c]:
                player.coo()

            if keystate[K_SPACE]:
                player.scatter()

        #draw the scene
        for layer in layers:
            dirty.extend(layer.draw(screen))

        pygame.display.update(dirty)

        #cap the framerate
        clock.tick_busy_loop(FPS)



#call the "main" function if running this script
if __name__ == '__main__': main()
